import 'package:flutter/services.dart';
import 'package:pixelpay_sdk/base/request_behaviour.dart';
import 'package:pixelpay_sdk/base/response.dart';
import 'package:pixelpay_sdk/exceptions/invalid_credentials_exception.dart';
import 'package:pixelpay_sdk/models/settings.dart';
import 'package:pixelpay_sdk/responses/error_response.dart';
import 'package:pixelpay_sdk/responses/failure_response.dart';
import 'package:pixelpay_sdk/responses/input_error_response.dart';
import 'package:pixelpay_sdk/responses/network_failure_response.dart';
import 'package:pixelpay_sdk/responses/no_access_response.dart';
import 'package:pixelpay_sdk/responses/not_found_response.dart';
import 'package:pixelpay_sdk/responses/payload_response.dart';
import 'package:pixelpay_sdk/responses/payment_declined_response.dart';
import 'package:pixelpay_sdk/responses/preconditional_response.dart';
import 'package:pixelpay_sdk/responses/success_response.dart';
import 'package:pixelpay_sdk/responses/timeout_response.dart';

class ServiceBehaviour {
  /// Settings service model
  Settings settings;

  /// Initialize service
  ServiceBehaviour(this.settings);

  /// Native Method Channel communicator
  static const MethodChannel channel = MethodChannel('pixelpay_sdk');

  /// Mapping and cast HTTP response
  ///
  /// @param body
  /// @param status
  /// @return
  Response parseResponse(String body, int status) {
    final response = Response.fromJson(body);
    response.setStatus(status);

    switch (status) {
      case 200:
        return SuccessResponse().fromResponse(response);

      case 202:
        return PayloadResponse().fromResponse(response);

      case 400:
        return ErrorResponse().fromResponse(response);

      case 401:
      case 403:
        return NoAccessResponse().fromResponse(response);

      case 402:
        return PaymentDeclinedResponse().fromResponse(response);

      case 404:
      case 405:
      case 406:
        return NotFoundResponse().fromResponse(response);

      case 408:
        return TimeoutResponse().fromResponse(response);

      case 412:
      case 418:
        return PreconditionalResponse().fromResponse(response);

      case 422:
        return InputErrorResponse().fromResponse(response);

      case 500:
        return FailureResponse().fromResponse(response);

      default:
        if (status > 500) {
          return NetworkFailureResponse().fromResponse(response);
        } else {
          return response;
        }
    }
  }

  /// Verify settings variables and inherith values
  ///
  /// @param method
  /// @param params
  /// @return
  Future<Response> send(String method, Map<String, dynamic> params) async {
    if (settings.auth_key == null || settings.auth_hash == null) {
      throw InvalidCredentialsException();
    }

    final Map<String, dynamic>? result =
        await ServiceBehaviour.channel.invokeMapMethod(method, params);

    return parseResponse(result!['response'], int.parse(result['status']));
  }

  /// Sets the SDK signature and returns the settings
  ///
  /// @param transaction The transaction with the signature information
  /// @param settings The settings object
  /// @return The settings object as a JSON string
  String getSettings(RequestBehaviour transaction, Settings settings) {
    settings.sdk = transaction.from;
    settings.sdk_version = transaction.sdk_version;

    return settings.toJson();
  }
}
